/* serial LCD routines support header	*/

#ifndef _LCD_h_
#define _LCD_h_

/* SPI interface definitions	*/
/* SSPSTAT REGISTER */

// Master SPI mode only

#define   SMPEND        0x80           // Input data sample at end of data out             
#define   SMPMID        0x00           // Input data sample at middle of data out

#define   MODE_00       0              // Setting for SPI bus Mode 0,0
//CKE           0x40                   // SSPSTAT register 
//CKP           0x00                   // SSPCON1 register 

#define   MODE_01       1              // Setting for SPI bus Mode 0,1
//CKE           0x00                   // SSPSTAT register 
//CKP           0x00                   // SSPCON1 register

#define   MODE_10       2              // Setting for SPI bus Mode 1,0
//CKE           0x40                   // SSPSTAT register
//CKP           0x10                   // SSPCON1 register

#define   MODE_11       3              // Setting for SPI bus Mode 1,1
//CKE           0x00                   // SSPSTAT register
//CKP           0x10                   // SSPCON1 register

/* SSPCON1 REGISTER */
#define   SSPENB        0x20           // Enable serial port and configures SCK, SDO, SDI

#define   SPI_FOSC_4    0              // SPI Master mode, clock = Fosc/4
#define   SPI_FOSC_16   1              // SPI Master mode, clock = Fosc/16
#define   SPI_FOSC_64   2              // SPI Master mode, clock = Fosc/64
#define   SPI_FOSC_TMR2 3              // SPI Master mode, clock = TMR2 output/2
#define   SLV_SSON      4              // SPI Slave mode, /SS pin control enabled
#define   SLV_SSOFF     5              // SPI Slave mode, /SS pin control disabled

/* HD44780 commands */
/* Function set	*/
#define Fset_8b_2l_5x7	0x38	/* Function set (8-bit interface, 2 lines, 5*7 Pixels) 	*/
#define Fset_8b_1l_5x7	0x30	/* Function set (8-bit interface, 1 line, 5*7 Pixels)		*/
#define	Fset_4b_2l_5x7	0x28	/* Function set (4-bit interface, 2 lines, 5*7 Pixels)	*/
#define	Fset_4b_1l_5x7	0x20	/* Function set (4-bit interface, 1 line, 5*7 Pixels)		*/

/* Entry modes could be confusing. Use entry mode 06 (power-on default) until comfortable */
#define	Entry_04				0x04	/* Decrement Address Counter, Display shift OFF	*/ 
#define Entry_05				0x05	/* Decrement Address Counter, Display shift ON	*/
#define	Entry_06				0x06	/* Increment Address Counter, Display shift OFF	*/
#define	Entry_07				0x07	/* Increment Address Counter, Display shift ON	*/

#define	Scroll_right		0x1e	/* Scroll display one character right (all lines) 		*/
#define Scroll_left			0x18	/* Scroll display one character left (all lines)			*/

#define	LCD_HOME			0x02	/* Home (move cursor to top/left character position)	*/
#define	Cursor_left			0x10	/* Move cursor one character left											*/
#define	Cursor_right		0x14	/* Move cursor one character right										*/
#define Cursor_uline		0x0e	/* Turn on visible underline cursor 									*/
#define Cursor_block		0x0f	/* Turn on visible blinking-block cursor							*/
#define Cursor_invis		0x0c	/* Make cursor invisible															*/

#define	Display_blank		0x08	/* Blank the display (without clearing)								*/
#define Display_restore		0x0c	/* Restore the display (with cursor hidden)						*/
#define	LCD_CLRSCR			0x01	/* Clear Screen																				*/

#define	SET_CURSOR			0x80	/* Set cursor position (Set_cursor+DDRAM address)			*/
#define	Set_CGRAM				0x40	/* Set pointer in character-generator RAM Set_CGRAM+(CGRAM address)	*/
/* Read commands are not supported */

/* LCD function prototypes */
#define SPI_74HC595_init() SPI_init(SPI_FOSC_4,MODE_00,SMPMID)
#define LCD_sendcmd(a)  {	CLR_RS;				\
							LCD_sendbyte(a);	\
						}
												
#define LCD_sendchar(a)	{	SET_RS;				\
							LCD_sendbyte(a);	\
						}
						
void SPI_init( BYTE sync_mode, BYTE bus_mode, BYTE smp_phase );
BYTE wr_SPI( BYTE data );

void LCD_init( void );
void SPI_to_74HC595( void );
void LCD_sendbyte( BYTE tosend );
void LCD_send_string( const rom char *str_ptr );
void LCD_send_ram_string (char *str_ptr);
void LCD_send_hexbyte( BYTE data );
void LCD_Home( void );
void LCD_second_row( void );

#endif // _LCD_h_
